
#define ALERT_PIN   12

#include "LT3964_1.h"
#include <Arduino.h>
#include <stdint.h>
#include <util/delay.h>
#include "Linduino.h"
#include "LT_I2C.h"
#include <math.h>



#define MID   1
#define LEFT  2
#define RIGHT 3

#define AC_ADDR_0      0x20
#define SC_ADDR_0      0x50
#define BC_ADDR        0x0C
#define TM_ADDR        0x73

#define CHANNEL0       0X00
#define CHANNEL1       0X01
#define CHANNEL2       0X02
#define CHANNEL3       0X03
#define CHANNEL4       0X04
#define CHANNEL5       0X05
#define CHANNEL6       0X06
#define CHANNEL7       0X07

#define CONFIG_LATCH   0x80
#define CONFIG_CLOCK   0x40
#define CONFIG_PULSE1  0x08
#define CONFIG_PULSE2  0x04
#define CONFIG_PHASE   0x01



//******************Misc. Functions******************
long readTerminal();
uint8_t readTerminalBinary(int size, uint8_t *stringVar);
uint16_t calcDuty (float percentage);
uint8_t calcAnalog( float percentage);
void printPaddedBinary(uint8_t value);


uint8_t icAddress = 0b0001;


boolean startflag = 0;




void setup()
{
  startflag = 0;
  uint8_t ADD = 0;
  uint8_t chan = 0;
  uint8_t set = 0b0000;

  TWSR = (HARDWARE_I2C_PRESCALER_1 & 0x03);       //! 1) set the TWI prescaler  (400kHz f_scl)
  TWBR = 12;                                      //! 2) set the TWI bit rate   (400kHz f_scl)

  quikeval_I2C_connect();
  pinMode(ALERT_PIN, INPUT);

  Serial.begin(115200);

  Serial.println(F("*** LT3964-1 Command Line Interface ***\n"));

  lt3964_i2c_write_3bytes(icAddress, 0x01, 0b00001100);     // Turn on Clockout of LT3964 while keeping channels disabled

  lt3964_i2c_write_3bytes(icAddress, 0x01, 0b00000000);     // Turn on channels of LT3964 while keeping clockout enabled
  
  lt3964_i2c_broadcast_write();
}

void loop()
{
  int input1 = 0;
  uint8_t input2 = 0;              // Used for ACM write user input
  uint8_t input3 = 0;
  uint8_t input32 = 0;
  uint8_t input4 = 0;
  uint8_t regVal = 0;
  uint8_t regVal2 = 0;
  uint8_t regVal3 = 0;
  uint8_t startFlag = 0;
  uint8_t channel = 0;          // Value 0-7 maps to Channel 1-8 SCM input
  uint8_t scm15 = 0;             //SCM 15 bit Voth, set to 4.5V or 9V
  uint8_t scm14 = 0;             // SCM bit 14  Vsth, set to 1V or 4V
  uint8_t scm9 = 0;              //SCM bit 9 Ao to set constant on/off
  uint8_t scmAE = 0;             // SCM bits  8 combined
  uint8_t scm131211 = 0;        // SCM bits 13, 12, and 11 combined (values 0-7)
  uint8_t scm8 = 0;             // SCM bit 8 (value 0 or 1)
  uint8_t addr = 0;             // BCM fault address read
  uint8_t chipsel = 0;
  int pwmInput = 0;

  if (!startflag)
  {
    delay(500);
    Serial.println(F("\nPress any key and then Enter to continue"));
    while (!Serial.available())
    {}
    readTerminal();
    delay(1500);

    startflag = 1;
  }

  Serial.println(F("\nType associated # followed by enter to select a register to access:\n"));
  Serial.println(F("LT3964-1 Functions"));
  Serial.println(F("  (1) Read Register Contents"));
  Serial.println(F("  (2) Modify CONFIG Register"));
  Serial.println(F("  (3) Modify PWM Registers"));
  Serial.println(F("  (4) Modify ADIM Register"));
  Serial.println(F("  (5) Modify STATEN Register"));
  Serial.println(F("  (6) Broadcast Mode Read/Write "));
  Serial.println(F("  (7) Clear Faults"));
  Serial.println(F("  (8) Change IC Address"));

  input1 = readTerminal();
  Serial.print(F("You Entered: "));
  Serial.println(input1);

  switch (input1)
  {
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 1://----------------------Read All Registers-----------------------------------------//
  {
    int readChoose;

    Serial.println(F("    (1) Read All Registers"));
    Serial.println(F("    (2) Read CONFIG Register"));
    Serial.println(F("    (3) Read Channel 1 PWM Registers"));
    Serial.println(F("    (4) Read Channel 2 PWM Registers"));
    Serial.println(F("    (5) Read Channel 1 ADIM Register"));
    Serial.println(F("    (6) Read Channel 2 ADIM Register"));
    Serial.println(F("    (7) Read STATEN Register"));
    Serial.println(F("    (8) Read STATUS Register"));
    Serial.println(F("    (9) Read IC Address"));
    Serial.println(F("    (0) Return to Main Menu"));

    readChoose = readTerminal();
    Serial.print(F("You Entered: "));
    Serial.println(readChoose);
    
    uint8_t config = 0;
    uint8_t ch1pwm1 = 0;
    uint8_t ch1pwm2 = 0;
    uint8_t ch2pwm1 = 0;
    uint8_t ch2pwm2 = 0;
    uint8_t adim1 = 0;
    uint8_t adim2 = 0;
    uint8_t staten = 0;
    uint8_t status = 0;

    lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &config);
    lt3964_i2c_read_bytes(icAddress, CH1PWM1, &ch1pwm1);
    lt3964_i2c_read_bytes(icAddress, CH1PWM2, &ch1pwm2);
    lt3964_i2c_read_bytes(icAddress, CH2PWM1, &ch2pwm1);
    lt3964_i2c_read_bytes(icAddress, CH2PWM2, &ch2pwm2);
    lt3964_i2c_read_bytes(icAddress, ADIM1, &adim1);
    lt3964_i2c_read_bytes(icAddress, ADIM2, &adim2);
    lt3964_i2c_read_bytes(icAddress, STATEN, &staten);
    lt3964_i2c_read_bytes(icAddress, STATUS, &status);

    switch(readChoose)
    {
      case 1:
      {
        Serial.println(F("Register Contents: \n"));
        Serial.print(F("CONFIG Reg: "));
        printPaddedBinary(config);
        Serial.print("\n");
        Serial.print(F("CH1PWM1 Reg: "));
        printPaddedBinary(ch1pwm1);
        Serial.print("\n");
        Serial.print(F("CH1PWM2 Reg: "));
        printPaddedBinary(ch1pwm2);
        Serial.print("\n");
        Serial.print(F("CH2PWM1 Reg: "));
        printPaddedBinary(ch2pwm1);
        Serial.print("\n");
        Serial.print(F("CH2PWM2 Reg: "));
        printPaddedBinary(ch2pwm2);
        Serial.print("\n");
        Serial.print(F("ADIM1 Reg: "));
        printPaddedBinary(adim1);
        Serial.print("\n");
        Serial.print(F("ADIM2 Reg: "));
        printPaddedBinary(adim2);
        Serial.print("\n");
        Serial.print(F("STATEN Reg: "));
        printPaddedBinary(staten);
        Serial.print("\n");
        Serial.print(F("STATUS Reg: "));
        printPaddedBinary(status);
        Serial.print("\n");
        break;
      }
      case 2:
      {
        Serial.print(F("CONFIG Reg: "));
        printPaddedBinary(config);
        Serial.print("\n");

        if((config & (1 << 7)) >> 7)
        {
          Serial.println(F("LATCHOFF Mode: ENABLED"));
        }
        else
        {
          Serial.println(F("LATCHOFF Mode: DISABLED"));
        }

        if((config & (1 << 6)) >> 6)
        {
          Serial.println(F("Clock Output: ENABLED"));
        }
        else
        {
          Serial.println(F("Clock Output: DISABLED"));
        }

        if((config & (1 << 3)) >> 3)
        {
          Serial.println(F("Channel 1 PWM: ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 1 PWM: DISABLED"));
        }

        if((config & (1 << 2)) >> 2)
        {
          Serial.println(F("Channel 2 PWM: ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 2 PWM: DISABLED"));
        }

        if((config & (1 << 0)) >> 0)
        {
          Serial.println(F("PWM 180 Degrees out of Phase"));
        }
        else
        {
          Serial.println(F("PWM in Phase"));
        }
        break;
      }
      case 3:
      {
        Serial.print(F("CH1PWM1 Reg: "));
        printPaddedBinary(ch1pwm1);
        Serial.print("\n");
        Serial.print(F("CH1PWM2 Reg: "));
        printPaddedBinary(ch1pwm2);
        Serial.print("\n");

        int scale1 = 0;
        if((ch1pwm1 & (1 << 7)) >> 7)
        {
          scale1 += 4;
        }
        if((ch1pwm1 & (1 << 6)) >> 6)
        {
          scale1 += 2;
        }
        if((ch1pwm1 & (1 << 5)) >> 5)
        {
          scale1 += 1;
        }
        Serial.print(F("Scale: "));
        Serial.println(scale1);

        float pwm1duty = 0;
        
        if((ch1pwm2 & (1 << 7)) >> 7)
        {
          pwm1duty += pow(2,(5 + scale1));
        }
        if((ch1pwm2 & (1 << 6)) >> 6)
        {
          pwm1duty += pow(2,(4 + scale1));
        }
        if((ch1pwm2 & (1 << 5)) >> 5)
        {
          pwm1duty += pow(2,(3 + scale1));
        }
        if((ch1pwm2 & (1 << 4)) >> 4)
        {
          pwm1duty += pow(2,(2 + scale1));
        }
        if((ch1pwm2 & (1 << 3)) >> 3)
        {
          pwm1duty += pow(2,(1 + scale1));
        }
        if((ch1pwm2 & (1 << 2)) >> 2)
        {
          pwm1duty += pow(2,(scale1));
        }
        if(((ch1pwm2 & (1 << 1)) >> 1) && scale1 >= 1)
        {
          pwm1duty += pow(2,(scale1 - 1));
        }
        if(((ch1pwm2 & (1 << 0)) >> 0) && scale1 >= 2)
        {
          pwm1duty += pow(2,(scale1 - 2));
        }
        if(((ch1pwm1 & (1 << 4)) >> 4) && scale1 >= 3)
        {
          pwm1duty += pow(2,(scale1 - 3));
        }
        if(((ch1pwm1 & (1 << 3)) >> 3) && scale1 >= 4)
        {
          pwm1duty += pow(2,(scale1 - 4));
        }
        if(((ch1pwm1 & (1 << 2)) >> 2) && scale1 >= 5)
        {
          pwm1duty += pow(2,(scale1 - 5));
        }
        if(((ch1pwm1 & (1 << 1)) >> 1) && scale1 >= 6)
        {
          pwm1duty += pow(2,(scale1 - 6));
        }
        if(((ch1pwm1 & (1 << 0)) >> 0) && scale1 >= 7)
        {
          pwm1duty += pow(2,(scale1 - 7));
        }
        if (pwm1duty > 0)
        {
          pwm1duty += 1; 
        }
        Serial.print(F("PWM On Time Value: "));
        Serial.println((int)pwm1duty);
        float percDC1 = 0;
        percDC1 = pwm1duty / (pow(2,(scale1+6)));
        percDC1 = percDC1 * 100;
        Serial.print(F("Duty Cycle: "));
        Serial.print(percDC1);
        Serial.println(F("%"));
        
        break;
      }
      case 4:
      {
        Serial.print(F("CH2PWM1 Reg: "));
        printPaddedBinary(ch2pwm1);
        Serial.print("\n");
        Serial.print(F("CH2PWM2 Reg: "));
        printPaddedBinary(ch2pwm2);
        Serial.print("\n");

        int scale2 = 0;
        if((ch2pwm1 & (1 << 7)) >> 7)
        {
          scale2 += 4;
        }
        if((ch2pwm1 & (1 << 6)) >> 6)
        {
          scale2 += 2;
        }
        if((ch2pwm1 & (1 << 5)) >> 5)
        {
          scale2 += 1;
        }
        Serial.print(F("Scale: "));
        Serial.println(scale2);

        float pwm2duty = 0;
        if((ch2pwm2 & (1 << 7)) >> 7)
        {
          pwm2duty += pow(2,(5 + scale2));
        }
        if((ch2pwm2 & (1 << 6)) >> 6)
        {
          pwm2duty += pow(2,(4 + scale2));
        }
        if((ch2pwm2 & (1 << 5)) >> 5)
        {
          pwm2duty += pow(2,(3 + scale2));
        }
        if((ch2pwm2 & (1 << 4)) >> 4)
        {
          pwm2duty += pow(2,(2 + scale2));
        }
        if((ch2pwm2 & (1 << 3)) >> 3)
        {
          pwm2duty += pow(2,(1 + scale2));
        }
        if((ch2pwm2 & (1 << 2)) >> 2)
        {
          pwm2duty += pow(2,(scale2));
        }
        if(((ch2pwm2 & (1 << 1)) >> 1) && scale2 >= 1)
        {
          pwm2duty += pow(2,(scale2 - 1));
        }
        if(((ch2pwm2 & (1 << 0)) >> 0) && scale2 >= 2)
        {
          pwm2duty += pow(2,(scale2 - 2));
        }
        if(((ch2pwm1 & (1 << 4)) >> 4) && scale2 >= 3)
        {
          pwm2duty += pow(2,(scale2 - 3));
        }
        if(((ch2pwm1 & (1 << 3)) >> 3) && scale2 >= 4)
        {
          pwm2duty += pow(2,(scale2 - 4));
        }
        if(((ch2pwm1 & (1 << 2)) >> 2) && scale2 >= 5)
        {
          pwm2duty += pow(2,(scale2 - 5));
        }
        if(((ch2pwm1 & (1 << 1)) >> 1) && scale2 >= 6)
        {
          pwm2duty += pow(2,(scale2 - 6));
        }
        if(((ch2pwm1 & (1 << 0)) >> 0) && scale2 >= 7)
        {
          pwm2duty += pow(2,(scale2 - 7));
        }
        if (pwm2duty > 0)
        {
          pwm2duty += 1; 
        }
        Serial.print(F("PWM On Time Value: "));
        Serial.println((int)pwm2duty);
        float percDC2 = 0;
        percDC2 = pwm2duty / (pow(2,(scale2+6)));
        percDC2 = percDC2 * 100;
        Serial.print(F("Duty Cycle: "));
        Serial.print(percDC2);
        Serial.println(F("%"));
        
        break;
      }
      case 5:
      {
        Serial.print(F("ADIM1 Reg: "));
        printPaddedBinary(adim1);
        Serial.print("\n");

        int adim1perc = 0;
        adim1perc = (adim1 * 100) / 255;
        Serial.print(F("Dimmed to "));
        Serial.print(adim1perc);
        Serial.println(F("% of Full Load"));

        break;
      }
      case 6:
      {
        Serial.print(F("ADIM2 Reg: "));
        printPaddedBinary(adim2);
        Serial.print("\n");

        int adim2perc = 0;
        adim2perc = (adim2 * 100) / 255;
        Serial.print(F("Dimmed to "));
        Serial.print(adim2perc);
        Serial.println(F("% of Full Load"));

        break;
      }
      case 7:
      {
        Serial.print(F("STATEN Reg: "));
        printPaddedBinary(staten);
        Serial.print("\n");

        if((staten & (1 << 7)) >> 7)
        {
          Serial.println(F("Channel 2 Overcurrent: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 2 Overcurrent: REPORTING DISABLED"));
        }

        if((staten & (1 << 6)) >> 6)
        {
          Serial.println(F("Channel 2 SHORTLED: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 2 SHORTLED: REPORTING DISABLED"));
        }

        if((staten & (1 << 5)) >> 5)
        {
          Serial.println(F("Channel 2 OPENLED: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 2 OPENLED: REPORTING DISABLED"));
        }

        if((staten & (1 << 4)) >> 4)
        {
          Serial.println(F("Channel 2 Overvoltage: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 2 Overvoltage: REPORTING DISABLED"));
        }

        if((staten & (1 << 3)) >> 3)
        {
          Serial.println(F("Channel 1 Overcurrent: REPORING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 1 Overcurrent: REPORTING DISABLED"));
        }

        if((staten & (1 << 2)) >> 2)
        {
          Serial.println(F("Channel 1 SHORTLED: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 1 SHORTLED: REPORTING DISABLED"));
        }

        if((staten & (1 << 1)) >> 1)
        {
          Serial.println(F("Channel 1 OPENLED: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 1 OPENLED: REPORTING DISABLED"));
        }

        if((staten & (1 << 0)) >> 0)
        {
          Serial.println(F("Channel 1 Overvoltage: REPORTING ENABLED"));
        }
        else
        {
          Serial.println(F("Channel 1 Overvoltage: REPORTING DISABLED"));
        }

        break;
      }
      case 8:
      {
        Serial.print(F("STATUS Reg: "));
        printPaddedBinary(status);
        Serial.print("\n");

        if((status & (1 << 7)) >> 7)
        {
          Serial.println(F("Channel 2 Overcurrent: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 2 Overcurrent: NO"));
        }

        if((status & (1 << 6)) >> 6)
        {
          Serial.println(F("Channel 2 SHORTLED: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 2 SHORTLED: NO"));
        }

        if((status & (1 << 5)) >> 5)
        {
          Serial.println(F("Channel 2 OPENLED: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 2 OPENLED: NO"));
        }

        if((status & (1 << 4)) >> 4)
        {
          Serial.println(F("Channel 2 Overvoltage: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 2 Overvoltage: NO"));
        }

        if((status & (1 << 3)) >> 3)
        {
          Serial.println(F("Channel 1 Overcurrent: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 1 Overcurrent: NO"));
        }

        if((status & (1 << 2)) >> 2)
        {
          Serial.println(F("Channel 1 SHORTLED: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 1 SHORTLED: NO"));
        }

        if((status & (1 << 1)) >> 1)
        {
          Serial.println(F("Channel 1 OPENLED: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 1 OPENLED: NO"));
        }

        if((status & (1 << 0)) >> 0)
        {
          Serial.println(F("Channel 1 Overvoltage: DETECTED"));
        }
        else
        {
          Serial.println(F("Channel 1 Overvoltage: NO"));
        }

        break;
      }
      case 9:
      {
        if(icAddress == 0b0001)
        {
          Serial.println("ADDR1 = Float, ADDR2 = 0");
          Serial.println("A3 A2 A1 A0: 0001");
        }
        else if(icAddress == 0b0011)
        {
          Serial.println("ADDR1 = 1, ADDR2 = 0");
          Serial.println("A3 A2 A1 A0: 0011");
        }
        else if(icAddress == 0b0100)
        {
          Serial.println("ADDR1 = 0, ADDR2 = Float");
          Serial.println("A3 A2 A1 A0: 0100");
        }
        else if(icAddress == 0b0101)
        {
          Serial.println("ADDR1 = Float, ADDR2 = Float");
          Serial.println("A3 A2 A1 A0: 0101");
        }
        else if(icAddress == 0b0111)
        {
          Serial.println("ADDR1 = 1, ADDR2 = Float");
          Serial.println("A3 A2 A1 A0: 0111");
        }
        else if(icAddress == 0b1100)
        {
          Serial.println("ADDR1 = 0, ADDR2 = 1");
          Serial.println("A3 A2 A1 A0: 1100");
        }
        else if(icAddress == 0b1101)
        {
          Serial.println("ADDR1 = Float, ADDR2 = 1");
          Serial.println("A3 A2 A1 A0: 1101");
        }
        else if(icAddress == 0b1111)
        {
          Serial.println("ADDR1 = 1, ADDR2 = 1");
          Serial.println("A3 A2 A1 A0: 1111");
        }
        else
        {
          Serial.println("Not writing to a valid IC address");
        }
        break;
      }
      case 0:
      {
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
    }
    break;
  }
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 2://----------------------Write to CONFIG Register LT3964-1----------------------//
  {
    int configChoose = 0;
    Serial.println(F("    (1) Modify CONFIG Register Byte"));
    Serial.println(F("    (2) Modify LATCHOFF Mode Enable Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (3) Modify Clock Output Enable Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (4) Modify CH1 Pulse Dim Enable Bit (0 = Disabled (Always On), 1 = Enabled)"));
    Serial.println(F("    (5) Modify CH2 Pulse Dim Enable Bit (0 = Disabled (Always On), 1 = Enabled)"));
    Serial.println(F("    (6) Modify PWM Dimming Phase Shift of CH1 and CH2 (0 = In Phase, 1 = 180 Degrees Out of Phase)"));
    Serial.println(F("    (0) Return to Main Menu"));


    configChoose = readTerminal();
    Serial.print(F("You Entered: "));
    Serial.println(configChoose);

    int en = 0;

    switch ((int) configChoose)
    {
    case 1:
    {
      uint8_t inputData = 0;
      Serial.print(F("Enter 8-bit binary value to write to CONFIG Register: "));
      if (!readTerminalBinary(8, &inputData))
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      
      lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, inputData);
      Serial.println("Entered!");
      Serial.println("CONFIG Register Updated!");
      break;
    }
    case 2:
    {
      uint8_t updateLatch = 0;
      lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &updateLatch);
      Serial.println(F("(0) Disable LATCHOFF Mode or (1) Enable LATCHOFF Mode or (2) Return to Main Menu?"));
      en = readTerminal();
      switch (en)
      {
      case 0:
      {
         updateLatch = (updateLatch & ~CONFIG_LATCH);
         lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateLatch);
         Serial.println(F("You Entered: 0"));
         Serial.println(F("LATCHOFF Mode: DISABLED"));
         break;
      }
      case 1:
      {
        updateLatch = (updateLatch | CONFIG_LATCH);
        lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateLatch);
        Serial.println(F("You Entered: 1"));
        Serial.println(F("LATCHOFF Mode: ENABLED"));
        break;
      }
      case 2:
      {
        Serial.println(F("You Entered: 2"));
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      }
      break;
    }
    case 3:
    {
      uint8_t updateClock = 0;
      lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &updateClock);
      Serial.println(F("(0) Disable CLOCK OUTPUT or (1) Enable CLOCK OUTPUT or (2) Return to Main Menu?"));
      en = readTerminal();
      switch (en)
      {
      case 0:
      {
         updateClock = (updateClock & ~CONFIG_CLOCK);
         lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateClock);
         Serial.println(F("You Enterd: 0"));
         Serial.println(F("CLOCK OUTPUT: DISABLED"));
         break;
      }
      case 1:
      {
        updateClock = (updateClock | CONFIG_CLOCK);
        lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateClock);
        Serial.println(F("You Entered: 1"));
        Serial.println(F("CLOCK OUTPUT: ENABLED"));
        break;
      }
      case 2:
      {
        Serial.println(F("You Entered: 2"));
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      }
      break;
    }
    case 4:
    {
      uint8_t updateDim1 = 0;
      lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &updateDim1);
      Serial.println(F("(0) Disable CH1 Pulse Dim (Always On) or (1) Enable CH1 Pulse Dim or (2) Return to Main Menu?"));
      en = readTerminal();
      switch (en)
      {
      case 0:
      {
         updateDim1 = (updateDim1 & ~CONFIG_PULSE1);
         lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateDim1);
         Serial.println(F("You Entered: 0"));
         Serial.println(F("CH1 PWM: OFF"));
         break;
      }
      case 1:
      {
        updateDim1 = (updateDim1 | CONFIG_PULSE1);
        lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateDim1);
        Serial.println(F("You Entered: 1"));
        Serial.println(F("CH1 PWM: ON"));
        break;
      }
      case 2:
      {
        Serial.println(F("You Entered: 2"));
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      }
      break;
    }
    case 5:
    {
      uint8_t updateDim2 = 0;
      lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &updateDim2);
      Serial.println(F("(0) Disable CH2 Pulse Dim (Always On) or (1) Enable CH2 Pulse Dim or (2) Return to Main Menu?"));
      en = readTerminal();
      switch (en)
      {
      case 0:
      {
         updateDim2 = (updateDim2 & ~CONFIG_PULSE2);
         lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateDim2);
         Serial.println(F("You Entered: 0"));
         Serial.println(F("CH2 PWM: OFF"));
         break;
      }
      case 1:
      {
        updateDim2 = (updateDim2 | CONFIG_PULSE2);
        lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updateDim2);
        Serial.println(F("You Entered: 1"));
        Serial.println(F("CH2 PWM: ON"));
        break;
      }
      case 2:
      {
        Serial.println(F("You Entered: 2"));
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      }
      break;
    }
    case 6:
    {
      uint8_t updatePhase = 0;
      lt3964_i2c_read_bytes(icAddress, CONFIG_REG, &updatePhase);
      Serial.println(F("PWM Dimming (0) in Phase or (1) 180 Degrees Out of Phase or (2) Return to Main Menu?"));
      en = readTerminal();
      switch (en)
      {
      case 0:
      {
         updatePhase = (updatePhase & ~CONFIG_PHASE);
         lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updatePhase);
         Serial.println(F("You Entered: 0"));
         Serial.println(F("PWM Dimming in Phase"));
         break;
      }
      case 1:
      {
        updatePhase = (updatePhase | CONFIG_PHASE);
        lt3964_i2c_write_3bytes(icAddress, CONFIG_REG, updatePhase);
        Serial.println(F("You Entered: 1"));
        Serial.println(F("PWM Dimming 180 Degrees Out of Phase"));
        break;
      }
      case 2:
      {
        Serial.println(F("You Entered: 2"));
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
      }
      break;
    }
    case 0:
    {
      Serial.println(F("You Entered: 0"));
      Serial.println(F("\n Returning to Main Menu\n"));
      break;
    }
    }
    break;    
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  case 3://---------------------------Modify PWM Registers------------------------------------//
  {
    
    Serial.println(F("Write to (1) CH1PWM1 and CH1PWM2 or (2) CH2PWM1 and CH2PWM2 or (0) return to main menu?"));
    pwmInput = readTerminal();

    Serial.print("You Entered: ");
    Serial.println(pwmInput);
    
    switch (pwmInput)
    {
      case 1: 
      {
        int pwm1choose = 0;
        Serial.println(F("Input (1) binary data for the registers or (2) guided values?"));
        pwm1choose = readTerminal();

        Serial.print("You Entered: ");
        Serial.println(pwm1choose);

        switch (pwm1choose)
        {
          case 1:
          {
            uint8_t pwmdatah = 0;
            uint8_t pwmdatal = 0;
            Serial.print(F("Enter 8-bit binary value to write to CH1PWM1 Register: "));
            if (!readTerminalBinary(8, &pwmdatal))
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            Serial.println("Entered!");
            Serial.print(F("Enter 8-bit binary value to write to CH1PWM2 Register: "));
            if (!readTerminalBinary(8, &pwmdatah))
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            Serial.println("Entered!");
            lt3964_i2c_write_3bytes(icAddress, CH1PWM1, pwmdatal);
            lt3964_i2c_write_3bytes(icAddress, CH1PWM2, pwmdatah);
            Serial.println("CH1PWM Registers Updated!");
            break;
          }
          case 2:
          {
            uint8_t c1p1 = 0;
            uint8_t c1p2 = 0;
            lt3964_i2c_read_bytes(icAddress, CH1PWM1, &c1p1);
            lt3964_i2c_read_bytes(icAddress, CH1PWM2, &c1p2);
            
            int scale = 0;
            long onTime = 0;
            long maximumOn = 0;
            Serial.print(F("Enter a SCALE value ranging from 0-7: "));
            scale = readTerminal();
            Serial.println(scale);
            if (scale < 0 || scale > 7)
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            
            //CH1PWM1 Bit 7
            if (scale >= 4)
            {
              c1p1 = (c1p1 | 0x80);
            }
            else
            {
              c1p1 = (c1p1 & ~0x80);
            }

            //CH1PWM1 Bit 6
            if (scale == 2 || scale == 3 || scale == 6 || scale == 7)
            {
              c1p1 = (c1p1 | 0x40);
            }
            else
            {
              c1p1 = (c1p1 & ~0x40);
            }

            //CH1PWM1 Bit 5
            if (scale % 2 == 1)
            {
              c1p1 = (c1p1 | 0x20);
            }
            else
            {
              c1p1 = (c1p1 & ~0x20);
            }
            
            maximumOn = pow(2, (scale + 6));
            Serial.print(F("Enter an on time ranging from 0 - "));
            Serial.print((int) maximumOn);
            Serial.print(": ");
            onTime = readTerminal();
            Serial.println(onTime);
            if (onTime < 0 || onTime > maximumOn)
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            
            //CH1PWM2 Bit 7
            if (onTime >= pow(2, (scale + 5)))
            {
              c1p2 = (c1p2 | 0x80);
              onTime -= pow(2, (scale + 5)); 
            }
            else
            {
              c1p2 = (c1p2 & ~0x80);
            }

            //CH1PWM2 Bit 6
            if (onTime >= pow(2, (scale + 4)))
            {
              c1p2 = (c1p2 | 0x40);
              onTime -= pow(2, (scale + 4)); 
            }
            else
            {
              c1p2 = (c1p2 & ~0x40);
            }

            //CH1PWM2 Bit 5
            if (onTime >= pow(2, (scale + 3)))
            {
              c1p2 = (c1p2 | 0x20);
              onTime -= pow(2, (scale + 3));
            }
            else
            {
              c1p2 = (c1p2 & ~0x20);  
            }

            //CH1PWM2 Bit 4
            if (onTime >= pow(2, (scale + 2)))
            {
              c1p2 = (c1p2 | 0x10);
              onTime -= pow(2, (scale + 2)); 
            }
            else
            {
              c1p2 = (c1p2 & ~0x10);
            }

            //CH1PWM2 Bit 3
            if (onTime >= pow(2, (scale + 1)))
            {
              c1p2 = (c1p2 | 0x08);
              onTime -= pow(2, (scale + 1)); 
            }
            else
            {
              c1p2 = (c1p2 & ~0x08);
            }

            //CH1PWM2 Bit 2
            if (onTime >= pow(2, scale))
            {
              c1p2 = (c1p2 | 0x04);
              onTime -= pow(2, scale); 
            }
            else
            {
              c1p2 = (c1p2 & ~0x04);
            }

            //CH1PWM2 Bit 1
            if ((scale >= 1) && (onTime >= pow(2, (scale - 1))))
            {
              c1p2 = (c1p2 | 0x02);
              onTime -= pow(2, (scale - 1));
            }
            else
            {
              c1p2 = (c1p2 & ~0x02);
            }

            //CH1PWM2 Bit 0
            if ((scale >= 2) && (onTime >= pow(2, (scale - 2))))
            {
              c1p2 = (c1p2 | 0x01);
              onTime -= pow(2, (scale - 2));
            }
            else
            {
              c1p2 = (c1p2 & ~0x01);
            }

            //CH1PWM1 Bit 4
            if ((scale >= 3) && (onTime >= pow(2, (scale - 3))))
            {
              c1p1 = (c1p1 | 0x10);
              onTime -= pow(2, (scale - 3));
            }
            else
            {
              c1p1 = (c1p1 & ~0x10);
            }

            //CH1PWM1 Bit 3
            if ((scale >= 4) && (onTime >= pow(2, (scale - 4))))
            {
              c1p1 = (c1p1 | 0x08);
              onTime -= pow(2, (scale - 4));
            }
            else
            {
              c1p1 = (c1p1 & ~0x08);
            }

            //CH1PWM1 Bit 2
            if ((scale >= 5) && (onTime >= pow(2, (scale - 5))))
            {
              c1p1 = (c1p1 | 0x04);
              onTime -= pow(2, (scale - 5));
            }
            else
            {
              c1p1 = (c1p1 & ~0x04);
            }

            //CH1PWM1 Bit 1
            if ((scale >= 6) && (onTime >= pow(2, (scale - 6))))
            {
              c1p1 = (c1p1 | 0x02);
              onTime -= pow(2, (scale - 6));
            }
            else
            {
              c1p1 = (c1p1 & ~0x02);
            }

            //CH1PWM1 Bit 0
            if ((scale == 7) && (onTime >= pow(2, (scale - 7))))
            {
              c1p1 = (c1p1 | 0x01);
              onTime -= pow(2, (scale - 7));
            }
            else
            {
              c1p1 = (c1p1 & ~0x01);
            }

            lt3964_i2c_write_3bytes(icAddress, CH1PWM1, c1p1);
            lt3964_i2c_write_3bytes(icAddress, CH1PWM2, c1p2);
            Serial.println(F("Channel 1 PWM has been updated"));

            break;
          }
        }
        break;
      }
      case 2:
      {
        int pwm2choose = 0;
        Serial.println(F("Input (1) binary data for the registers or (2) guided values?"));
        pwm2choose = readTerminal();

        Serial.print("You Entered: ");
        Serial.println(pwm2choose);
        
        switch (pwm2choose)
        {
          case 1:
          {
            uint8_t pwmdatah = 0;
            uint8_t pwmdatal = 0;
            Serial.print(F("Enter 8-bit binary value to write to CH2PWM1 Register: "));
            if (!readTerminalBinary(8, &pwmdatal))
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            Serial.println("Entered!");
            Serial.print(F("Enter 8-bit binary value to write to CH2PWM2 Register: "));
            if (!readTerminalBinary(8, &pwmdatah))
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            Serial.println("Entered!");
            lt3964_i2c_write_3bytes(icAddress, CH2PWM1, pwmdatal);
            lt3964_i2c_write_3bytes(icAddress, CH2PWM2, pwmdatah);
            Serial.println("CH2PWM Registers Updated!");
            break;
          }
          case 2:
          {
            uint8_t c2p1 = 0;
            uint8_t c2p2 = 0;
            lt3964_i2c_read_bytes(icAddress, CH2PWM1, &c2p1);
            lt3964_i2c_read_bytes(icAddress, CH2PWM2, &c2p2);
            
            int scale = 0;
            long onTime = 0;
            long maximumOn = 0;
            Serial.print("Enter a SCALE value ranging from 0-7: ");
            scale = readTerminal();
            Serial.println(scale);
            if (scale < 0 || scale > 7)
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            
            //CH1PWM1 Bit 7
            if (scale >= 4)
            {
              c2p1 = (c2p1 | 0x80);
            }
            else
            {
              c2p1 = (c2p1 & ~0x80);
            }

            //CH1PWM1 Bit 6
            if (scale == 2 || scale == 3 || scale == 6 || scale == 7)
            {
              c2p1 = (c2p1 | 0x40);
            }
            else
            {
              c2p1 = (c2p1 & ~0x40);
            }

            //CH1PWM1 Bit 5
            if (scale % 2 == 1)
            {
              c2p1 = (c2p1 | 0x20);
            }
            else
            {
              c2p1 = (c2p1 & ~0x20);
            }
            
            maximumOn = pow(2, (scale + 6));
            Serial.print(F("Enter an on time ranging from 0 - "));
            Serial.print((int) maximumOn);
            Serial.print(": ");
            onTime = readTerminal();
            Serial.println(onTime);
            if (onTime < 0 || onTime > maximumOn)
            {
              Serial.println(F("\n***********Invalid input***********\n"));
              break;
            }
            
            //CH1PWM2 Bit 7
            if (onTime >= pow(2, (scale + 5)))
            {
              c2p2 = (c2p2 | 0x80);
              onTime -= pow(2, (scale + 5)); 
            }
            else
            {
              c2p2 = (c2p2 & ~0x80);
            }

            //CH1PWM2 Bit 6
            if (onTime >= pow(2, (scale + 4)))
            {
              c2p2 = (c2p2 | 0x40);
              onTime -= pow(2, (scale + 4)); 
            }
            else
            {
              c2p2 = (c2p2 & ~0x40);
            }

            //CH1PWM2 Bit 5
            if (onTime >= pow(2, (scale + 3)))
            {
              c2p2 = (c2p2 | 0x20);
              onTime -= pow(2, (scale + 3));
            }
            else
            {
              c2p2 = (c2p2 & ~0x20);  
            }

            //CH1PWM2 Bit 4
            if (onTime >= pow(2, (scale + 2)))
            {
              c2p2 = (c2p2 | 0x10);
              onTime -= pow(2, (scale + 2)); 
            }
            else
            {
              c2p2 = (c2p2 & ~0x10);
            }

            //CH1PWM2 Bit 3
            if (onTime >= pow(2, (scale + 1)))
            {
              c2p2 = (c2p2 | 0x08);
              onTime -= pow(2, (scale + 1)); 
            }
            else
            {
              c2p2 = (c2p2 & ~0x08);
            }

            //CH1PWM2 Bit 2
            if (onTime >= pow(2, scale))
            {
              c2p2 = (c2p2 | 0x04);
              onTime -= pow(2, scale); 
            }
            else
            {
              c2p2 = (c2p2 & ~0x04);
            }

            //CH1PWM2 Bit 1
            if ((scale >= 1) && (onTime >= pow(2, (scale - 1))))
            {
              c2p2 = (c2p2 | 0x02);
              onTime -= pow(2, (scale - 1));
            }
            else
            {
              c2p2 = (c2p2 & ~0x02);
            }

            //CH1PWM2 Bit 0
            if ((scale >= 2) && (onTime >= pow(2, (scale - 2))))
            {
              c2p2 = (c2p2 | 0x01);
              onTime -= pow(2, (scale - 2));
            }
            else
            {
              c2p2 = (c2p2 & ~0x01);
            }

            //CH1PWM1 Bit 4
            if ((scale >= 3) && (onTime >= pow(2, (scale - 3))))
            {
              c2p1 = (c2p1 | 0x10);
              onTime -= pow(2, (scale - 3));
            }
            else
            {
              c2p1 = (c2p1 & ~0x10);
            }

            //CH1PWM1 Bit 3
            if ((scale >= 4) && (onTime >= pow(2, (scale - 4))))
            {
              c2p1 = (c2p1 | 0x08);
              onTime -= pow(2, (scale - 4));
            }
            else
            {
              c2p1 = (c2p1 & ~0x08);
            }

            //CH1PWM1 Bit 2
            if ((scale >= 5) && (onTime >= pow(2, (scale - 5))))
            {
              c2p1 = (c2p1 | 0x04);
              onTime -= pow(2, (scale - 5));
            }
            else
            {
              c2p1 = (c2p1 & ~0x04);
            }

            //CH1PWM1 Bit 1
            if ((scale >= 6) && (onTime >= pow(2, (scale - 6))))
            {
              c2p1 = (c2p1 | 0x02);
              onTime -= pow(2, (scale - 6));
            }
            else
            {
              c2p1 = (c2p1 & ~0x02);
            }

            //CH1PWM1 Bit 0
            if ((scale == 7) && (onTime >= pow(2, (scale - 7))))
            {
              c2p1 = (c2p1 | 0x01);
              onTime -= pow(2, (scale - 7));
            }
            else
            {
              c2p1 = (c2p1 & ~0x01);
            }

            lt3964_i2c_write_3bytes(icAddress, CH2PWM1, c2p1);
            lt3964_i2c_write_3bytes(icAddress, CH2PWM2, c2p2);
            Serial.println(F("Channel 2 PWM has been updated"));

            break;
          }
        }
        break;
      }
      case 0:
      {
        Serial.println(F("\n Returning to Main Menu \n"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n")); 
        break;
      }
      break;      
    }
    break; 
  }
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 4://----------------------Write to ADIM Register LT3964-1----------------------//
  {
    uint8_t channelsel = 0;
    Serial.println(F("Write to (1) CH1 or (2) CH2 or (3) both or (0) return to main menu?"));
    channelsel = readTerminal();
    Serial.print("You Entered: ");
    Serial.println(channelsel);
    if ((channelsel > 4) || (channelsel < 0))
    {
      Serial.println(F("\n***********Invalid input***********\n"));
      break;
    }
    if (channelsel == 0)
    {
      Serial.println(F("\n Returning to Main Menu \n"));
      break;
    }
    uint8_t inputData = 0;
    int inputInt = 0;
    Serial.print(F("Enter a value between 0 (0% of full load) and 255 (100% of full load) to write to ADIM Register(s): "));
    inputData = readTerminal();

    Serial.println(inputData);
    
    switch (channelsel)
    {
    case 1:
    {
      lt3964_i2c_write_3bytes(icAddress, ADIM1, inputData);
      Serial.println("ADIM Register Updated!");
      break;
    }
    case 2:
    {
      lt3964_i2c_write_3bytes(icAddress, ADIM2, inputData);
      Serial.println("ADIM Register Updated!");
      break;
    }
    case 3:
    {
      lt3964_i2c_write_3bytes(icAddress, ADIM1, inputData);
      lt3964_i2c_write_3bytes(icAddress, ADIM2, inputData);
      Serial.println("ADIM Register Updated!");
      break;
    }
    }
    break;
  }
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 5://----------------------Write to STATEN Register LT3964-1----------------------//
  {
    int statSel = 0;
    Serial.println(F("    (1) Modify STATEN Register Byte"));
    Serial.println(F("    (2) Modify CH2 Overcurrent Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (3) Modify CH2 Short LED Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (4) Modify CH2 Open LED Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (5) Modify CH2 Overvoltage Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (6) Modify CH1 Overcurrent Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (7) Modify CH1 Short LED Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (8) Modify CH1 Open LED Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (9) Modify CH1 Overvoltage Reporting Bit (0 = Disabled, 1 = Enabled)"));
    Serial.println(F("    (0) Return to Main Menu"));
    
    statSel = readTerminal();
    switch (statSel)
    {
      case 1:
      {
        uint8_t inputData = 0;
        Serial.print(F("Enter 8-bit binary value to write to STATEN Register: "));
        Serial.println("Entered!");
        if (!readTerminalBinary(8, &inputData))
        {
          Serial.println(F("\n\n Returning to Main Menu"));
          break;
        }
        lt3964_i2c_write_3bytes(icAddress, STATEN, inputData);
        Serial.println("STATEN Register Updated!");
        break;
      }
      case 2:
      {
        uint8_t updateCH2oc = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH2oc);
        Serial.println(F("CH2 overcurrent reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH2oc = (updateCH2oc & ~0X80);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2oc);
            Serial.println(F("CH2 overcurrent reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH2oc = (updateCH2oc | 0X80);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2oc);
            Serial.println(F("CH2 overcurrent reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }

      case 3:
      {
        uint8_t updateCH2short = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH2short);
        Serial.println(F("CH2 Short LED reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH2short = (updateCH2short & ~0X40);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2short);
            Serial.println(F("CH2 Short LED reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH2short = (updateCH2short | 0X40);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2short);
            Serial.println(F("CH2 Short LED reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 4:
      {
        uint8_t updateCH2open = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH2open);
        Serial.println(F("CH2 open LED reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH2open = (updateCH2open & ~0X20);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2open);
            Serial.println(F("CH2 open LED reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH2open = (updateCH2open | 0X20);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2open);
            Serial.println(F("CH2 open LED reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 5:
      {
        uint8_t updateCH2ov = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH2ov);
        Serial.println(F("CH2 overvoltage reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH2ov = (updateCH2ov & ~0X10);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2ov);
            Serial.println(F("CH2 overvoltage reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH2ov = (updateCH2ov | 0X10);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH2ov);
            Serial.println(F("CH2 overvoltage reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 6:
      {
        uint8_t updateCH1oc = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH1oc);
        Serial.println(F("CH1 overcurrent reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH1oc = (updateCH1oc & ~0X08);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1oc);
            Serial.println(F("CH1 overcurrent reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH1oc = (updateCH1oc | 0X08);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1oc);
            Serial.println(F("CH1 overcurrent reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 7:
      {
        uint8_t updateCH1short = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH1short);
        Serial.println(F("CH1 short LED reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH1short = (updateCH1short & ~0X04);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1short);
            Serial.println(F("CH1 short LED reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH1short = (updateCH1short | 0X04);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1short);
            Serial.println(F("CH1 short LED reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 8:
      {
        uint8_t updateCH1open = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH1open);
        Serial.println(F("CH1 open LED reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH1open = (updateCH1open & ~0X02);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1open);
            Serial.println(F("CH1 open LED reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH1open = (updateCH1open | 0X02);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1open);
            Serial.println(F("CH1 open LED reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 9:
      {
        uint8_t updateCH1ov = 0;
        lt3964_i2c_read_bytes(icAddress, STATEN, &updateCH1ov);
        Serial.println(F("CH1 overvoltage reporting (0) Disabled or (1) Enabled or (2) Return to Main Menu?"));
        int enableStatBit = 0;
        enableStatBit = readTerminal();
        switch (enableStatBit)
        {
          case 0:
          {
            updateCH1ov = (updateCH1ov & ~0X01);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1ov);
            Serial.println(F("CH1 overvoltage reporting on ALERT pin: DISABLED"));
            break;
          }
          case 1:
          {
            updateCH1ov = (updateCH1ov | 0X01);
            lt3964_i2c_write_3bytes(icAddress, STATEN, updateCH1ov);
            Serial.println(F("CH1 overvoltage reporting on ALERT pin: ENABLED"));
            break;
          }
          case 2:
          {
            Serial.println(F("\n Returning to Main Menu"));
            break;
          }
          default:
          {
            Serial.println(F("\n***********Invalid input***********\n"));
            break;
          }
        }
        break;
      }
      case 0:
      {
        Serial.println(F("\n\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********\n"));
        break;
      }
    }     
    break;
  }
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 6://----------------------Broadcast Mode Read/Write----------------------//
  {
    char choose = 0;
    Serial.println(F("\nWould you like to (W) Broadcast Mode Write or (R) Broadcast Mode Read or (0) Return to Main Menu?"));
    while ( Serial.available() == 0);
    while ( Serial.available() > 0 )
    {
      choose = Serial.read();
    }
    Serial.print(F("You Entered: "));
    Serial.println(choose);
    switch (choose)
    {
      case 'W':
      {
        Serial.println(F("\nBroadcast Mode (BCM) Write to Synchronize PWM of Multiple Addresses "));
        lt3964_i2c_broadcast_write();
        Serial.print(F("\n"));
        Serial.println(F("Broadcast write complete - synchronized!\n"));
        break;
      }
      case 'w':
      {
        Serial.println(F("\nBroadcast Mode (BCM) Write to Synchronize PWM of Multiple Addresses "));
        lt3964_i2c_broadcast_write();
        Serial.print(F("\n"));
        Serial.println(F("Broadcast write complete - synchronized!\n"));
        break;
      }
      case 'R':
      {
        uint8_t readStat = 0;
        int flag = 0;
        for (uint8_t i = 1; i < 16; i++) 
        {
          readStat = 0;
          lt3964_i2c_read_bytes(i, STATUS, &readStat);
          if (readStat != 0)
          {
            flag = 1;
          }
        }
        for (uint8_t j = 1; j < 16; j++)
        {
          lt3964_i2c_write_3bytes(j, STATUS, 0b000000000);
        }
        Serial.println(F("\nBroadcast Mode Read (Check Fault Address) "));
        if (flag == 1)
        {
          while (lt3964_i2c_broadcast_read(&addr) == 0)
          {
            Serial.print(F("\nFaulty Device Address: "));
            printPaddedBinary(addr);
            if((addr & (1 << 1)) >> 1)
            {
              if((addr & (1 << 2)) >> 2)
              {
                Serial.print("\nADDR1 = 1, ");
              }
              else
              {
                Serial.print("\nADDR1 = Float, ");
              }
            }
            else
            {
              Serial.print("\nADDR1 = 0, ");
            }
            if((addr & (1 << 3)) >> 3)
            {
              if((addr & (1 << 4)) >> 4)
              {
                Serial.println("ADDR2 = 1");
              }
              else
              {
                Serial.println("ADDR2 = Float");
              }
            }
            else
            {
              Serial.println("ADDR2 = 0");
            }
          }
        }
        else
        {
          Serial.println(F("\nNo LT3964-1 Devices Reporting Fault"));          
        }
        flag = 0;
        break;
      }
      case 'r':
      {
        uint8_t readStat = 0;
        int flag = 0;
        for (uint8_t i = 1; i < 16; i++) 
        {
          readStat = 0;
          lt3964_i2c_read_bytes(i, STATUS, &readStat);
          if (readStat != 0)
          {
            flag = 1;
          }
        }
        for (uint8_t j = 1; j < 16; j++)
        {
          lt3964_i2c_write_3bytes(j, STATUS, 0b000000000);
        }
        Serial.println(F("\nBroadcast Mode Read (Check Fault Address) "));
        if (flag == 1)
        {
          while (lt3964_i2c_broadcast_read(&addr) == 0)
          {
            Serial.print(F("\nFaulty Device Address: "));
            printPaddedBinary(addr);
            if((addr & (1 << 1)) >> 1)
            {
              if((addr & (1 << 2)) >> 2)
              {
                Serial.print("\nADDR1 = 1, ");
              }
              else
              {
                Serial.print("\nADDR1 = Float, ");
              }
            }
            else
            {
              Serial.print("\nADDR1 = 0, ");
            }
            if((addr & (1 << 3)) >> 3)
            {
              if((addr & (1 << 4)) >> 4)
              {
                Serial.println("ADDR2 = 1");
              }
              else
              {
                Serial.println("ADDR2 = Float");
              }
            }
            else
            {
              Serial.println("ADDR2 = 0");
            }
          }
        }
        else
        {
          Serial.println(F("\nNo LT3964-1 Devices Reporting Fault"));          
        }
        flag = 0;
        break;
      }
      case '0':
      {
        Serial.println(F("\n Returning to Main Menu"));
        break;
      }
      default:
      {
        Serial.println(F("\n***********Invalid input***********"));
        break;
      }
    }    
    break;
  } 
  ////////////////////////////////////////////////////////////////////////////////////////////
  case 7://----------------------Clear All Faults LT3964-1----------------------//
  {
    char yesno = 0;
    Serial.print("\n");
    Serial.println(F("Clear STATUS Reg Contents? (Y)es / (N)o"));
    while ( Serial.available() == 0);
    while ( Serial.available() > 0 )
    {
      yesno = Serial.read();
    }
    switch (yesno)
    {
    case 'Y':
    {
      lt3964_i2c_write_3bytes(icAddress, STATUS, 0b000000000);
      Serial.println(F("STATUS Register Cleared!"));
      break;
    }
    case 'y':
    {
      lt3964_i2c_write_3bytes(icAddress, STATUS, 0b000000000);
      Serial.println(F("STATUS Register Cleared!"));
      break;
    }
    case 'N':
    {
      Serial.println(F("STATUS Register NOT Cleared"));
      break;
    }
    case 'n':
    {
      Serial.println(F("STATUS Register NOT Cleared"));
      break;
    }
    }
    Serial.print("\n");
    break;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  case 8:
  {
    int addr1sel = 0;
    int addr2sel = 0;
    
    Serial.println("Would you like to set ADDR1 to (0) 0, (1) 1, or (2) Float?"  );
    addr1sel = readTerminal();
    switch (addr1sel)
    {
      case 0:
      {
        icAddress = icAddress & 0b1100;
        Serial.println(F("ADDR1 set to 0"));
        break;
      }
      case 1:
      {
        icAddress = icAddress | 0b0011;
        Serial.println(F("ADDR1 set to 1"));
        break;
      }
      case 2:
      {
        icAddress = icAddress | 0b0001;
        icAddress = icAddress & 0b1101;
        Serial.println(F("ADDR1 set to Float"));
        break;
      }
    }

    Serial.println("\nWould you like to set ADDR2 to (0) 0, (1) 1, or (2) Float?"  );
    addr2sel = readTerminal();
    switch (addr2sel)
    {
      case 0:
      {
        icAddress = icAddress & 0b0011;
        Serial.println(F("ADDR2 set to 0"));
        break;
      }
      case 1:
      {
        icAddress = icAddress | 0b1100;
        Serial.println(F("ADDR2 set to 1"));
        break;
      }
      case 2:
      {
        icAddress = icAddress | 0b0100;
        icAddress = icAddress & 0b0111;
        Serial.println(F("ADDR2 set to Float"));
        break;
      }
    }

    Serial.println(F("\n    IC Address has been Updated!\n"));
    break;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  default:
    Serial.println(F("\n***********Invalid input***********"));
    break;
  }
  ////////////////////////////////////////////////////////////////////////////////////////////
}


//************************************************************************************
// Functions for command line interface. NOT TO BE INCLUDED IN LT3967 HEADER FILE

long readTerminal()
{
  long buf;
  while ( Serial.available() == 0);
  while ( Serial.available() > 0)
  {
    buf = Serial.parseInt();
  }
  return (buf);
}

uint8_t readTerminalBinary(int size, uint8_t *stringVar)
{
  char bufStr[] = "00000000";
  uint8_t value = 0;
  uint8_t length = 0;
  int x = 0;

  while ( Serial.available() == 0);
  while ( Serial.available() > 0 )
  {
    length = Serial.readBytes(bufStr, size);
    delay(1);

    if (x != 0 || length != size)
    {
      return (0);
    }
    x = 1;
  }

  for (int x = 0; x < size; x++)
  {
    if (bufStr[x] == '1')
    {
      double temp = pow(2, 7 - x) + 0.5;
      value += (uint8_t) temp;
    }
    else if (bufStr[x] != '0')
    {
      return (0);
    }
  }

  value = (value >> (8 - size));
  *stringVar = value;
  return (1);
}


void printPaddedBinary(uint8_t value)
{
  uint8_t mask = 0b10000000;
  while (mask != 0)
  {
    if ((mask & value) != mask)
    {
      Serial.print("0");
    }
    else
    {
      Serial.print("1");
    }
    mask = mask >> 1;
  }
  return;
}
