//#include <Arduino.h>
//#include "Linduino.h"
//#include "LT_I2C.h"
//#include "LT3964.h"
#include <stdint.h>



#ifndef LT3964_H
#define LT3964_H



//************LT3964 Command Registers***************
#define CONFIG_REG   0x01
#define CH1PWM1   0x02
#define CH1PWM2   0x03
#define CH2PWM1   0x04
#define CH2PWM2   0x05
#define ADIM1       0x06
#define ADIM2       0x07
#define STATEN      0x08
#define STATUS      0x09

//************LT3964 Read-Only Registers***************
#define PARTNUM1    0xFD
#define PARTNUM2    0xFE
#define PARTNUM3    0xFF

//************LT3964 7-BIT Read/Write/BC***************
#define I2C_WRITE   0x60
#define I2C_READ    0x60
#define I2C_BCAST   0xC0

#define FREQ_SCALAR_7 0xE0
#define FREQ_SCALAR_6 0xC0
#define FREQ_SCALAR_5 0xA0
#define FREQ_SCALAR_4 0x80
#define FREQ_SCALAR_3 0x60
#define FREQ_SCALAR_2 0x40
#define FREQ_SCALAR_1 0x20
#define FREQ_SCALAR_0 0x00


#define I2C_READ_BIT    0x01
#define I2C_WRITE_BIT   0x00

//**************************************************************************************
// Send out three bytes over I2C transmission lines
// This is useful for sending a single word of data to a single sub address register
// used by the LT3964
int8_t lt3964_i2c_write_3bytes(uint8_t address, uint8_t reg, uint8_t data);


//**************************************************************************************
// Send  out five bytes over I2C transmission lines
// This is useful for sending data to the two PWM sub address registers
// with corresponding data for each reg
int8_t lt3964_i2c_read_bytes(uint8_t address, uint8_t reg, uint8_t *value);

//**************************************************************************************
// Sends  Broadcast Mode Address to all LT3964 connected to the I2C bus
// to check which device has an alert. Function argument #2 (value) returns
// the 4-BIT address of the LT3964 asserting the fault condition
int8_t lt3964_i2c_broadcast_read(uint8_t *value);


//**************************************************************************************
// Sends  Broadcast Write to all LT3964 connected to the I2C bus
// to synchronize PWM dimming cycles amoung synced slave LT3964s
int8_t lt3964_i2c_broadcast_write(void);


#endif
